@@ -12,6 +12,7 @@
 #if HAVE_EXIF
 #include <libexif/exif-data.h>
 #include <libexif/exif-utils.h>
+#include <libexif/exif-loader.h>
 #endif
 
 #include "libeog-marshal.h"
@@ -19,6 +20,7 @@
 #include "eog-image-private.h"
 #include "eog-pixbuf-util.h"
 #include "eog-image-jpeg.h"
+#include "eog-image-cache.h"
 
 static GThread     *thread                     = NULL;
 static gboolean     thread_running             = FALSE;
@@ -290,16 +292,13 @@ eog_image_dispose (GObject *object)
 
 	priv = EOG_IMAGE (object)->priv;
 
+	eog_image_free_mem (EOG_IMAGE (object));
+
 	if (priv->uri) {
 		gnome_vfs_uri_unref (priv->uri);
 		priv->uri = NULL;
 	}
 
-	if (priv->image) {
-		g_object_unref (priv->image);
-		priv->image = NULL;
-	}
-
 	if (priv->caption) {
 		g_free (priv->caption);
 		priv->caption = NULL;
@@ -328,13 +327,6 @@ eog_image_dispose (GObject *object)
 		g_list_free (priv->undo_stack);
 		priv->undo_stack = NULL;
 	}
-
-#if HAVE_EXIF
-	if (priv->exif) {
-		exif_data_unref (priv->exif);
-		priv->exif = NULL;
-	}
-#endif
 }
 
 static void
@@ -494,6 +486,7 @@ eog_image_new_uri (GnomeVFSURI *uri)
 
 	priv->uri = gnome_vfs_uri_ref (uri);
 	priv->mode = EOG_IMAGE_LOAD_DEFAULT;
+	priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 	priv->modified = FALSE;
 	priv->load_thread = NULL;
 
@@ -717,124 +710,6 @@ update_exif_data (EogImage *image)
 	}	
 }
 
-#define JPEG_MARKER_SOI  0xd8
-#define	JPEG_MARKER_APP0 0xe0
-#define JPEG_MARKER_APP1 0xe1
-
-typedef enum {
-	EXIF_LOADER_READ,
-	EXIF_LOADER_READ_SIZE_HIGH_BYTE,
-	EXIF_LOADER_READ_SIZE_LOW_BYTE,
-	EXIF_LOADER_SKIP_BYTES,
-	EXIF_LOADER_EXIF_FOUND,
-	EXIF_LOADER_FAILED
-} ExifLoaderState;
-
-typedef struct {
-	ExifLoaderState state;
-	ExifData *data;
-	int      size;
-	int      last_marker;
-	guchar  *buffer_data;
-	int      bytes_read;
-} ExifLoaderData;
-
-/* This function imitates code from libexif, written by Lutz
- * Mller. See libexif/exif-data.c:exif_data_new_from_file. Here, it
- * can cope with a sequence of data chunks.
- */
-static gboolean
-exif_loader_write (ExifLoaderData *eld, guchar *buffer, int len)
-{
-	int i;
-	int len_remain;
-
-	if (eld->state == EXIF_LOADER_FAILED) return FALSE;
-	if (eld->state == EXIF_LOADER_EXIF_FOUND && eld->data != NULL) return FALSE;
-
-	for (i = 0; (i < len) && eld->state != EXIF_LOADER_EXIF_FOUND && eld->state != EXIF_LOADER_FAILED; i++) {
-
-		switch (eld->state) {
-		case EXIF_LOADER_SKIP_BYTES:
-			eld->size--;
-			if (eld->size == 0) {
-				eld->state = EXIF_LOADER_READ;
-			}
-			break;
-			
-		case EXIF_LOADER_READ_SIZE_HIGH_BYTE:
-			eld->size = buffer [i] << 8;
-			eld->state = EXIF_LOADER_READ_SIZE_LOW_BYTE;
-			break;
-			
-		case EXIF_LOADER_READ_SIZE_LOW_BYTE:
-			eld->size |= buffer [i];
-			eld->size = eld->size - 2;
-			
-			switch (eld->last_marker) {
-			case JPEG_MARKER_APP0:
-				eld->state = EXIF_LOADER_SKIP_BYTES;
-				break;
-
-			case JPEG_MARKER_APP1:
-				eld->state = EXIF_LOADER_EXIF_FOUND;
-				break;
-
-			default:
-				g_assert_not_reached ();
-			}
-
-			eld->last_marker = 0;
-			break;
-
-		default:
-			if (buffer[i] != 0xff) {
-				if (buffer [i] == JPEG_MARKER_APP0 ||
-				    buffer [i] == JPEG_MARKER_APP1) 
-				{
-					eld->state = EXIF_LOADER_READ_SIZE_HIGH_BYTE;
-					eld->last_marker = buffer [i];
-				}
-				else if (buffer [i] != JPEG_MARKER_SOI) {
-					eld->state = EXIF_LOADER_FAILED;
-				}
-			}
-		}
-
-	}
-	
-	len_remain = len - i;
-
-	if (eld->state == EXIF_LOADER_EXIF_FOUND && len_remain > 0) {
-
-		if (eld->buffer_data == NULL) {
-			eld->buffer_data = g_new0 (guchar, eld->size);
-			eld->bytes_read = 0;
-		}
-
-		if (eld->bytes_read < eld->size) {
-			int cp_len;
-			int rest;
-
-			/* the number of bytes we need to copy */
-			rest = eld->size - eld->bytes_read;
-			cp_len = MIN (rest, len_remain);
-			
-			g_assert ((cp_len + eld->bytes_read) <= eld->size);
-
-			/* copy memory */
-			memcpy (eld->buffer_data + eld->bytes_read, &buffer[i], cp_len);
-
-			eld->bytes_read += cp_len;
-		}
-
-		if (eld->bytes_read == eld->size) {
-			eld->data = exif_data_new_from_data (eld->buffer_data, eld->size);
-		}
-	}
-
-	return (eld->data != NULL);
-}
 
 #endif 
 
@@ -854,7 +729,7 @@ real_image_load (gpointer data)
 	gboolean failed;
 	GError *error = NULL;
 #if HAVE_EXIF
-	ExifLoaderData *eld;
+	ExifLoader *exif_loader;
 #endif
 
 	img = EOG_IMAGE (data);
@@ -905,8 +780,7 @@ real_image_load (gpointer data)
 	loader = gdk_pixbuf_loader_new ();
 	failed = FALSE;
 #if HAVE_EXIF
-	eld = g_new0 (ExifLoaderData, 1);
-	eld->state = EXIF_LOADER_READ;
+	exif_loader = exif_loader_new ();
 #endif
 
 	g_signal_connect_object (G_OBJECT (loader), "size-prepared", (GCallback) load_size_prepared, img, 0);
@@ -939,12 +813,14 @@ real_image_load (gpointer data)
 		}
 
 #if HAVE_EXIF
-		if (exif_loader_write (eld, buffer, bytes_read)) {
+		if (exif_loader != NULL && (exif_loader_write (exif_loader, buffer, bytes_read) == 0)) {
 			g_mutex_lock (img->priv->status_mutex);
-			exif_data_ref (eld->data);
-			priv->exif = eld->data;
+			priv->exif = exif_loader_get_data (exif_loader);
 			priv->load_status |= EOG_IMAGE_LOAD_STATUS_INFO_DONE;
 			g_mutex_unlock (img->priv->status_mutex);
+			
+			exif_loader_unref (exif_loader);
+			exif_loader = NULL;
 		}
 #endif
 	}
@@ -970,6 +846,7 @@ real_image_load (gpointer data)
 		else {
 			priv->error_message = NULL;
 		}
+		priv->status = EOG_IMAGE_STATUS_FAILED;
 		g_mutex_unlock (priv->status_mutex);
 	}
 	else if (priv->cancel_loading) {
@@ -978,10 +855,17 @@ real_image_load (gpointer data)
 			g_object_unref (priv->image);
 			priv->image = NULL;
 		}
-		priv->cancel_loading = TRUE;
+#if HAVE_EXIF
+		if (priv->exif != NULL) {
+			exif_data_unref (priv->exif);
+			priv->exif = NULL;
+		}
+#endif
+		priv->cancel_loading = FALSE;
 		priv->progress = 0.0;
 
 		priv->load_status |= EOG_IMAGE_LOAD_STATUS_CANCELLED;
+		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 		g_mutex_unlock (priv->status_mutex);
 	}
 	else {
@@ -1015,21 +899,17 @@ real_image_load (gpointer data)
 #if HAVE_EXIF
 		update_exif_data (img);
 #endif 
+		eog_image_cache_add (img);
+
 		if (trans != NULL && priv->mode == EOG_IMAGE_LOAD_PROGRESSIVE) {
 			priv->load_status |= EOG_IMAGE_LOAD_STATUS_TRANSFORMED;
 		}
+
 		priv->load_status |= EOG_IMAGE_LOAD_STATUS_DONE;
+		priv->status = EOG_IMAGE_STATUS_LOADED;
 		g_mutex_unlock (priv->status_mutex);
 	}
 
-#if HAVE_EXIF
-	if (eld->buffer_data != NULL)
-		g_free (eld->buffer_data);
-	if (eld->data)
-		exif_data_unref (eld->data);
-	g_free (eld);
-#endif
-	
 	if (error != NULL) {
 		g_error_free (error);
 	}
@@ -1046,27 +926,21 @@ void
 eog_image_load (EogImage *img, EogImageLoadMode mode)
 {
 	EogImagePrivate *priv;
-	gboolean image_loaded;
-	gboolean thread_running;
 
 	g_return_if_fail (EOG_IS_IMAGE (img));
 
 	priv = EOG_IMAGE (img)->priv;
 
-	g_return_if_fail (priv->uri != NULL);
-
-	g_mutex_lock (priv->status_mutex);
-	image_loaded = priv->image != NULL;
-	thread_running = priv->load_thread != NULL;
-	g_mutex_unlock (priv->status_mutex);
-
-	if (image_loaded && !thread_running) {
+	if (priv->status == EOG_IMAGE_STATUS_LOADED) {
 		g_assert (priv->image != NULL);
+		eog_image_cache_reload (img);
 		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FINISHED], 0);
 		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_INFO_FINISHED], 0);
 	}
-	else if (!image_loaded && !thread_running) { 
-                /* we assume that in all other cases there is no image loaded */
+	else if (priv->status == EOG_IMAGE_STATUS_FAILED) {
+		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FAILED], 0, "");		
+	}
+	else if (priv->status == EOG_IMAGE_STATUS_UNKNOWN) { 
 		g_assert (priv->image == NULL);
 		
                 /* make sure the object isn't destroyed while we try to load it */
@@ -1116,9 +990,9 @@ eog_image_load (EogImage *img, EogImageLoadMode mode)
 		}
 
 		/* start the thread machinery */
-		priv->load_id = g_timeout_add (CHECK_LOAD_TIMEOUT, check_load_status, img);
+		priv->status      = EOG_IMAGE_STATUS_LOADING;
+		priv->load_id     = g_timeout_add (CHECK_LOAD_TIMEOUT, check_load_status, img);
 		priv->load_thread = g_thread_create (real_image_load, img, TRUE, NULL);
-		
 	}
 }
 
@@ -1522,30 +1396,46 @@ eog_image_get_caption (EogImage *img)
 	return priv->caption;
 }
 
-void
-eog_image_free_mem (EogImage *img)
+void 
+eog_image_free_mem_private (EogImage *image)
 {
 	EogImagePrivate *priv;
 	
-	g_return_if_fail (EOG_IS_IMAGE (img));
-	
-	priv = img->priv;
+	priv = image->priv;
 
-	if (priv->image != NULL) {
-		gdk_pixbuf_unref (priv->image);
-		priv->image = NULL;
+	if (priv->status == EOG_IMAGE_STATUS_LOADING) {
+		eog_image_cancel_load (image);
 	}
-
+	else {
+		if (priv->image != NULL) {
+			gdk_pixbuf_unref (priv->image);
+			priv->image = NULL;
+		}
+		
 #if HAVE_EXIF
-	if (priv->exif != NULL) {
-		exif_data_unref (priv->exif);
-		priv->exif = NULL;
-	}
+		if (priv->exif != NULL) {
+			exif_data_unref (priv->exif);
+			priv->exif = NULL;
+		}
 #endif
 
-	priv->load_status = EOG_IMAGE_LOAD_STATUS_NONE;
+		priv->load_status = EOG_IMAGE_LOAD_STATUS_NONE;
+		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
+	}
 }
 
+void
+eog_image_free_mem (EogImage *image)
+{
+	g_return_if_fail (EOG_IS_IMAGE (image));
+
+	if (image->priv->image != NULL) {
+		eog_image_cache_remove (image);
+		eog_image_free_mem_private (image);
+	}
+}
+
+
 const gchar*        
 eog_image_get_collate_key (EogImage *img)
 {
@@ -1569,21 +1459,16 @@ void
 eog_image_cancel_load (EogImage *img)
 {
 	EogImagePrivate *priv;
-	gboolean join_thread = FALSE;
 
 	g_return_if_fail (EOG_IS_IMAGE (img));
 	
 	priv = img->priv;
 
 	g_mutex_lock (priv->status_mutex);
-	if (priv->load_thread != NULL) {
+	if (priv->status == EOG_IMAGE_STATUS_LOADING) {
 		priv->cancel_loading = TRUE;
-		join_thread = TRUE;
 	}
 	g_mutex_unlock (priv->status_mutex);
-
-	if (join_thread)
-		g_thread_join (priv->load_thread);
 }
 
 gpointer
@@ -1617,7 +1502,7 @@ eog_image_is_loaded (EogImage *img)
 	priv = img->priv;
 
 	g_mutex_lock (priv->status_mutex);
-	result = (priv->load_thread == NULL && priv->image != NULL);
+	result = (priv->status == EOG_IMAGE_STATUS_LOADED);
 	g_mutex_unlock (priv->status_mutex);
 	
 	return result;
