@@ -40,6 +40,7 @@
 #include "eog-file-selection.h"
 #include "eog-full-screen.h"
 #include "eog-hig-dialog.h"
+#include "eog-config-keys.h"
 
 #include <libgpi/gpi-dialog-pixbuf.h>
 #include <libgpi/gpi-mgr-pixbuf.h>
@@ -174,15 +175,16 @@ verb_FullScreen_cb (BonoboUIComponent *uic, gpointer data, const char *name)
 {
 	EogImageView *image_view;
 	GtkWidget *fs;
-	GList *list = NULL;
+	EogImageList *list;
 
 	g_return_if_fail (EOG_IS_IMAGE_VIEW (data));
 
 	image_view = EOG_IMAGE_VIEW (data);
 
-	list = g_list_prepend (list, EOG_IMAGE (image_view->priv->image));
+	list = eog_image_list_new ();
+	eog_image_list_add_image (list, EOG_IMAGE (image_view->priv->image));
 	fs = eog_full_screen_new (list, NULL);
-	g_list_free (list);
+	g_object_unref (list);
 
 	gtk_widget_show_all (fs);
 }
@@ -434,6 +436,12 @@ eog_image_view_create_ui (EogImageView *image_view)
 	bonobo_ui_component_add_verb_list_with_data (priv->uic, eog_verbs,
 						     image_view);
 
+	/* special case for SUN keyboards where F11 key has different keysym */
+	if (eog_full_screen_enable_SunF36()) {
+		bonobo_ui_component_add_verb (priv->uic, "FullScreenHiddenSun",
+					      verb_FullScreen_cb, image_view);
+	}
+
 	if (!priv->has_zoomable_frame) {
 		bonobo_ui_util_set_ui (priv->uic, DATADIR, "eog-image-view-ctrl-ui.xml", "EOG", NULL);
 
@@ -508,22 +516,17 @@ eog_image_view_get_prop (BonoboPropertyBag *bag,
 
 		zoom = floor (100 * eog_scroll_view_get_zoom (EOG_SCROLL_VIEW (priv->widget)));
 
-		if (priv->image == NULL) {
-			text = g_strdup (" ");
+		if (priv->image != NULL) {
+			eog_image_get_size (priv->image, &width, &height);
+		}
+		
+		if ((width > 0) && (height > 0)) {
+			/* [image width] x [image height] pixel  [zoom in percent] */
+			text = g_strdup_printf (_("%i x %i pixel    %i%%"), 
+						width, height, zoom);
 		}
 		else {
-			text = g_new0 (char, 40);
-			
-			eog_image_get_size (priv->image, &width, &height);
-
-			if ((width > 0) && (height > 0)) {
-				g_snprintf (text, 39, "%i x %i pixel    %i%%",
-					    width, height,
-					    zoom);
-			}
-			else {
-				g_snprintf (text, 39, "%i%%", zoom);
-			}
+			text = g_strdup (" ");
 		}
 
 		BONOBO_ARG_SET_STRING (arg, text);
@@ -724,7 +727,7 @@ transparency_changed_cb (GConfClient *client,
 		char *color_str;
 
 		color_str = gconf_client_get_string (priv->client,
-						     GCONF_EOG_VIEW_TRANS_COLOR, NULL);
+						     EOG_CONF_VIEW_TRANS_COLOR, NULL);
 		if (gdk_color_parse (color_str, &color)) {
 			eog_scroll_view_set_transparency (EOG_SCROLL_VIEW (priv->widget),
 							  TRANSP_COLOR, &color);
@@ -753,7 +756,7 @@ trans_color_changed_cb (GConfClient *client,
 
 	priv = EOG_IMAGE_VIEW (user_data)->priv;
 
-	value = gconf_client_get_string (priv->client, GCONF_EOG_VIEW_TRANSPARENCY, NULL);
+	value = gconf_client_get_string (priv->client, EOG_CONF_VIEW_TRANSPARENCY, NULL);
 
 	if (g_strcasecmp (value, "COLOR") != 0) return;
 
@@ -1167,23 +1170,23 @@ init_gconf_defaults (EogImageView *view)
 
 	priv->client = gconf_client_get_default ();
 	gconf_client_add_dir (priv->client,
-			      GCONF_EOG_VIEW_DIR,
+			      EOG_CONF_VIEW_DIR,
 			      GCONF_CLIENT_PRELOAD_ONELEVEL,
 			      NULL);
 
 
 	/* get preference values from gconf */
 	eog_scroll_view_set_antialiasing (EOG_SCROLL_VIEW (priv->widget),
-					  gconf_client_get_bool (priv->client, GCONF_EOG_VIEW_INTERP_TYPE, NULL));
+					  gconf_client_get_bool (priv->client, EOG_CONF_VIEW_INTERPOLATE, NULL));
 
 	transp_str = gconf_client_get_string (priv->client,
-					      GCONF_EOG_VIEW_TRANSPARENCY, NULL);
+					      EOG_CONF_VIEW_TRANSPARENCY, NULL);
 	if (g_strcasecmp (transp_str, "COLOR") == 0) {
 		GdkColor color;
 		char *color_str;
 
 		color_str = gconf_client_get_string (priv->client,
-						     GCONF_EOG_VIEW_TRANS_COLOR, NULL);
+						     EOG_CONF_VIEW_TRANS_COLOR, NULL);
 		if (gdk_color_parse (color_str, &color)) {
 			eog_scroll_view_set_transparency (EOG_SCROLL_VIEW (priv->widget),
 							  TRANSP_COLOR, &color);
@@ -1201,17 +1204,17 @@ init_gconf_defaults (EogImageView *view)
 	/* add gconf listeners */
 	priv->interp_type_notify_id =
 		gconf_client_notify_add (priv->client,
-					 GCONF_EOG_VIEW_INTERP_TYPE,
+					 EOG_CONF_VIEW_INTERPOLATE,
 					 interp_type_changed_cb,
 					 view, NULL, NULL);
 	priv->transparency_notify_id =
 		gconf_client_notify_add (priv->client,
-					 GCONF_EOG_VIEW_TRANSPARENCY,
+					 EOG_CONF_VIEW_TRANSPARENCY,
 					 transparency_changed_cb,
 					 view, NULL, NULL);
 	priv->trans_color_notify_id =
 		gconf_client_notify_add (priv->client,
-					 GCONF_EOG_VIEW_TRANS_COLOR,
+					 EOG_CONF_VIEW_TRANS_COLOR,
 					 trans_color_changed_cb,
 					 view, NULL, NULL);
 }
